use std::{fs::File, io::BufReader};

#[derive(serde::Deserialize, Debug)]
pub(crate) struct Config {
    pub(crate) matrix: MatrixConfig,
    pub(crate) meshtastic: MeshtasticConfig,
    #[serde(default = "get_db_uri")]
    pub(crate) db: String,
}

#[derive(serde::Deserialize, Debug)]
pub(crate) struct MatrixConfig {
    pub(crate) username: String,
    #[serde(default = "get_matrix_password")]
    pub(crate) password: String,
    #[serde(default = "get_device_name")]
    pub(crate) device_name: String,
    pub(crate) room: String,
}

#[derive(serde::Deserialize, Debug)]
pub(crate) struct MeshtasticConfig {
    #[serde(default = "get_meshtastic_address")]
    pub(crate) address: String, // expected format is host:port
}

fn get_device_name() -> String {
    "meshtastic-bridge".to_string()
}

fn get_db_uri() -> String {
    "sqlite://matrix-meshtastic-bridge.db".to_string()
}

fn get_matrix_password() -> String {
    match std::env::var("MATRIX_PASSWORD") {
        Ok(p) => p,
        Err(_) => "".to_string(),
    }
}

fn get_meshtastic_address() -> String {
    match std::env::var("MESHTASTIC_ADDRESS") {
        Ok(p) => p,
        Err(_) => "meshtastic.local".to_string(),
    }
}

pub(crate) async fn read_config() -> Config {
    let filename = match std::env::var("CONFIG_PATH") {
        Ok(value) => value,
        Err(_) => "config.json".to_string(),
    };

    log::info!("loading config file from {}", filename);

    let file = File::open(filename).expect("failed to read config file");
    let reader = BufReader::new(file);

    // Read the JSON contents of the file as an instance of `User`.
    serde_json::from_reader(reader).expect("failed to parse config.json")
}
